function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var React = require('react');
var React__default = _interopDefault(React);

var hasOwnProperty = Object.prototype.hasOwnProperty;
function is(x, y) {
    if (x === y) {
        return x !== 0 || y !== 0 || 1 / x === 1 / y;
    } else {
        return x !== x && y !== y;
    }
}

function shallowEqual(objA, objB) {
    if (is(objA, objB)) {
        return true;
    }
    if (typeof objA !== "object" || objA === null || typeof objB !== "object" || objB === null) {
        return false;
    }
    var keysA = Object.keys(objA);
    var keysB = Object.keys(objB);
    if (keysA.length !== keysB.length) {
        return false;
    }
    for (var i = 0;i < keysA.length; i++) {
        if (!hasOwnProperty.call(objB, keysA[i]) || !is(objA[keysA[i]], objB[keysA[i]])) {
            return false;
        }
    }
    return true;
}

var StoreContext = React__default.createContext();
var StoreProvider = StoreContext.Provider;
function useStore(mapState) {
    var store = React.useContext(StoreContext);
    var ref = React.useState(function () {
        var state = store.getState();
        return mapState(state);
    });
    var mappedState = ref[0];
    var setMappedState = ref[1];
    var prevMappedStateRef = usePreviousRef(mappedState);
    React.useEffect(function () {
        var checkForUpdates = function () {
            var nextState = store.getState();
            var nextMappedState = mapState(nextState);
            if (shallowEqual(prevMappedStateRef.current, nextMappedState)) {
                return;
            }
            setMappedState(nextMappedState);
        };
        checkForUpdates();
        return store.subscribe(checkForUpdates);
    }, [store,mapState]);
    return [mappedState,store.dispatch];
}

function usePreviousRef(value) {
    var ref = React.useRef();
    React.useEffect(function () {
        ref.current = value;
    });
    return ref;
}

exports.StoreProvider = StoreProvider;
exports.useStore = useStore;
